<?php

namespace Annie\Causes\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Annie\Causes\Http\Requests\CauseRequest;
use Annie\Causes\Interfaces\CauseRepositoryInterface;
use Annie\Causes\Models\Cause;

/**
 * Resource controller class for cause.
 */
class CauseResourceController extends BaseController
{

    /**
     * Initialize cause resource controller.
     *
     * @param type CauseRepositoryInterface $cause
     *
     * @return null
     */
    public function __construct(CauseRepositoryInterface $cause)
    {
        parent::__construct();
        $this->repository = $cause;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Annie\Causes\Repositories\Criteria\CauseResourceCriteria::class);
    }

    /**
     * Display a list of cause.
     *
     * @return Response
     */
    public function index(CauseRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Annie\Causes\Repositories\Presenter\CausePresenter::class)
                ->$function();
        }

        $causes = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('causes::cause.names'))
            ->view('causes::cause.index', true)
            ->data(compact('causes', 'view'))
            ->output();
    }

    /**
     * Display cause.
     *
     * @param Request $request
     * @param Model   $cause
     *
     * @return Response
     */
    public function show(CauseRequest $request, Cause $cause)
    {

        if ($cause->exists) {
            $view = 'causes::cause.show';
        } else {
            $view = 'causes::cause.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('causes::cause.name'))
            ->data(compact('cause'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new cause.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CauseRequest $request)
    {

        $cause = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('causes::cause.name')) 
            ->view('causes::cause.create', true) 
            ->data(compact('cause'))
            ->output();
    }

    /**
     * Create new cause.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CauseRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $cause                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('causes::cause.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('causes/cause/' . $cause->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/causes/cause'))
                ->redirect();
        }

    }

    /**
     * Show cause for editing.
     *
     * @param Request $request
     * @param Model   $cause
     *
     * @return Response
     */
    public function edit(CauseRequest $request, Cause $cause)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('causes::cause.name'))
            ->view('causes::cause.edit', true)
            ->data(compact('cause'))
            ->output();
    }

    /**
     * Update the cause.
     *
     * @param Request $request
     * @param Model   $cause
     *
     * @return Response
     */
    public function update(CauseRequest $request, Cause $cause)
    {
        try {
            $attributes = $request->all();

            $cause->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('causes::cause.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('causes/cause/' . $cause->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('causes/cause/' . $cause->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the cause.
     *
     * @param Model   $cause
     *
     * @return Response
     */
    public function destroy(CauseRequest $request, Cause $cause)
    {
        try {

            $cause->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('causes::cause.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('causes/cause/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('causes/cause/' . $cause->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple cause.
     *
     * @param Model   $cause
     *
     * @return Response
     */
    public function delete(CauseRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('causes::cause.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('causes/cause'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/causes/cause'))
                ->redirect();
        }

    }

    /**
     * Restore deleted causes.
     *
     * @param Model   $cause
     *
     * @return Response
     */
    public function restore(CauseRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('causes::cause.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/causes/cause'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/causes/cause/'))
                ->redirect();
        }

    }

}
